import os
import uuid
import time

from multiprocessing import Queue
from multiprocessing import Pool

import os
import subprocess
import shutil
from PyPDF2 import PdfFileWriter
from PyPDF2 import PdfFileReader

__pdf_filenames = Queue()
__text_filenames = Queue()


def convert_document(pdf_filename,
                     resolution=200,
                     delete_files=True,
                     page_delineation='\n--------\n',
                     verbose=False,
                     temp_dir=str(uuid.uuid4()),password='',
                     thumb_prefix='thumb_page_',
                     pool_count=2):
    just_pdf_filename = os.path.basename(pdf_filename)
    temp_pdf_filename = '{0}/{1}'.format(temp_dir, just_pdf_filename)
    shutil.copyfile(pdf_filename, temp_pdf_filename)
    filename = decrypt_pdf(temp_pdf_filename, temp_dir, password)
    filenames = split_pdf(filename, temp_dir)
    for filename in filenames:
        __pdf_filenames.put(filename)
    pool = Pool()
    pool.map_async(
        _yapot_worker,
        [(tid, just_pdf_filename, temp_dir, resolution) for
            tid in range(0, pool_count)],
    )
    while __text_filenames.qsize() != len(filenames):
        time.sleep(1)
    text_filenames = []
    try:
        while(1):
            text_filenames.append(__text_filenames.get_nowait())
    except:
        pass
    text = build_output_text(text_filenames, page_delineation)
    if delete_files:
        cleanup_yapot(temp_dir)
    return text


def _yapot_worker(args):
    tid, pdf_filename, temp_dir, resolution = args
    while(1):
        if not __pdf_filenames.qsize():
            break
        try:
            filename = __pdf_filenames.get_nowait()
        except:
            #  No more files in the queue
            break;
        image_filename = '{0}.tiff'.format(filename)
        success = pdf_to_image(filename, image_filename, resolution)
        try:
            text = image_ocr(image_filename)
        except Exception as e:
            #  TODO: Do something with this error ...
            pass
        text_filename = '{0}.txt'.format(image_filename)
        with open(text_filename,'w') as f:
            f.write(text)
        __text_filenames.put(text_filename)


def decrypt_pdf(pdf_filename, temp_dir, password):
    '''
    Some PDFs are encrypted.  This decrypts it.
    '''
    pdf_filename_unsecured = '{0}_unsecured.pdf'.format(
        pdf_filename
    )
    with open(os.devnull, 'w') as FNULL:
        cli = [
            'qpdf',
            '--password={0}'.format(password),
            '--decrypt',
            pdf_filename,
            pdf_filename_unsecured,
        ]
        subprocess.call(cli)
    return pdf_filename_unsecured


def split_pdf(pdf_filename, temp_dir):
    '''
    Split the PDF into n PDFs ( one for each page ).
    '''
    filenames = []
    inputpdf = PdfFileReader(open(pdf_filename, "rb"))
    if inputpdf.getIsEncrypted():
        inputpdf.decrypt('')
    for i in range(inputpdf.numPages):
        output = PdfFileWriter()
        output.addPage(inputpdf.getPage(i))
        filename = os.path.basename(pdf_filename)
        filename = "{0}/{1}-p{2}.pdf".format(temp_dir, filename, i)
        with open(filename, "wb") as outputStream:
            output.write(outputStream)
        filenames.append(filename)

    return filenames


def pdf_to_image(pdf_filename, image_filename, resolution):
    '''
    Converts a single PDF to a TIFF image.
    '''
    with open(os.devnull, 'w') as FNULL:
        cli = [
            'convert',
            '-depth',
            '8',
            '-density',
            '{0}'.format(resolution),
            '-threshold',
            '50%',
            '{0}'.format(pdf_filename),
            '-channel',
            'r',
            '-separate',
            '{0}'.format(image_filename),
        ]
        subprocess.call(cli)
    return None


def make_thumb(image_filename, thumb_filename, thumb_size):
    '''
    Makes a thumbnail from a larger image.
    '''
    with open(os.devnull, 'w') as FNULL:
        cli = [
            'convert',
            '-resize',
            '{0}x{0}'.format(thumb_size),
            image_filename,
            thumb_filename,
        ]
        subprocess.call(cli)
    return None


def image_ocr(filename):
    '''
    Uses tesseract to OCR an image.
    '''
    text = ''
    text_filename = '{0}.txt'.format(filename)
    with open(os.devnull, 'w') as FNULL:
        cli = [
            'tesseract',
            filename,
            filename,
            '-psm',
            '6',
        ]
        subprocess.call(cli)
    with open(text_filename, 'r') as f:
        text = f.read()
    return text


def build_output_text(filenames, page_delineation):
    '''
    Concatenate output text products into a single text blob.
    '''
    text = ''
    ordered_filenames = []
    for i in range(0,len(filenames)):
        ordered_filenames.append('')
    for filename in filenames:
        index = int(filename.split('.pdf.tiff.txt')[0].split('-p')[-1])
        ordered_filenames[index] = filename
    for filename in ordered_filenames:
        with open(filename) as f:
            contents = f.read()
        text += '{0}{1}'.format(contents, page_delineation)
    return text


def cleanup_yapot(output_dir):
    '''
    Clean up all the temp files that yapot makes.
    '''
    shutil.rmtree(output_dir)




import os
import subprocess

import shutil

#import PyPDF2 as pyPdf

from PyPDF2 import PdfFileWriter
from PyPDF2 import PdfFileReader

def decrypt_pdf(pdf_filename, password):

    pdf_filename_unsecured = '{0}_unsecured.pdf'.format(pdf_filename)

    with open(os.devnull, 'w') as FNULL:
        cmd = [
            'qpdf',
            '--password={0}'.format(password),
            '--decrypt',
            pdf_filename,
            pdf_filename_unsecured,
        ]
        #stdout=FNULL,
        #stderr=FNULL, #subprocess.STDOUT,
        subprocess.call(cmd)
    return pdf_filename_unsecured

def split_pdf(pdf_filename):

    filenames = []
    inputpdf = PdfFileReader(open(pdf_filename, "rb"))
    if inputpdf.getIsEncrypted():
        inputpdf.decrypt('')
    for i in range(inputpdf.numPages):
        output = PdfFileWriter()
        output.addPage(inputpdf.getPage(i))
        directory = os.path.dirname(pdf_filename)
        if directory == '':
            directory = '.'
        filename = os.path.basename(pdf_filename)
        filename = "{0}/{1}-p{2}.pdf".format(directory,filename,i)
        with open(filename, "wb") as outputStream:
            output.write(outputStream)
        filenames.append(filename)

    return filenames

def pdf_to_image(pdf_filename, image_filename, resolution):

    with open(os.devnull, 'w') as FNULL:

        cli = [
            'convert',
            '-depth',
            '8',
            '-density',
            '{0}'.format(resolution),
            '-threshold',
            '50%',
            '{0}'.format(pdf_filename),
            '-channel',
            'r',
            '-separate',
            '{0}'.format(image_filename),
        ]
        subprocess.call(cli)


        ''''
        cli = [
            'convert',
            '-antialias',
            '-density',
            '300',
            '-black-threshold',
            '50%',
            pdf_filename,
            image_filename,
        ]
        '''

        '''
        shutil.copyfile(pdf_filename, '{0}_inter.pdf'.format(pdf_filename))

        cli = [
            'convert',
            '-density',
            '{0}'.format(resolution),
            '-threshold',
            '50%',
            pdf_filename,
            pdf_filename,
        ]
        print ' '.join(cli)
        subprocess.call(cli)

        cli = [
            'gs',
            '-q',
            '-dQUIET',
            '-dSAFER',
            '-dBATCH',
            '-dNOPAUSE',
            '-dNOPROMPT',
            '-sDEVICE=tiffg4',
            '-r{0}x{0}'.format(resolution),
            '-sOutputFile={0}'.format(image_filename),
            pdf_filename
        ]
        print ' '.join(cli)
        subprocess.call(cli)
     
        #shutil.copyfile(image_filename, '{0}_inter.tiff'.format(image_filename))
        '''

        # print 'PDF converted.'
        

        '''
        cli2 = [
            'convert',
            '-antialias',
            '-black-threshold',
            '50%',
            image_filename,
            image_filename,
        ] 
        print ' '.join(cli2)
        subprocess.call(cli2)

        print "Image converted."
        '''

        '''
        subprocess.call(
            [
                'gs',
                '-q',
                '-dQUIET',
                '-dSAFER',
                '-dBATCH',
                '-dNOPAUSE',
                '-dNOPROMPT',
                '-sDEVICE=tiffg4',
                '-r{0}'.format(resolution),
                '-sOutputFile={0}'.format(image_filename),
                pdf_filename,
            ],
        )
        '''

    return None

def make_thumb(image_filename, thumb_filename, thumb_size):

    with open(os.devnull, 'w') as FNULL:
        subprocess.call(
            [
                'convert',
                '-resize',
                '{0}x{0}'.format(thumb_size),
                image_filename,
                thumb_filename,
            ],
        )

    return None

def image_ocr(filename):

    text = ''
    text_filename = '{0}.txt'.format(filename)
    with open(os.devnull, 'w') as FNULL:
        cmd = [
            'tesseract',
            filename,
            filename,
            '-psm',
            '6',
        ]

        #print ' '.join(cmd)
        subprocess.call(cmd)
    with open(text_filename, 'r') as f:
        text = f.read()

    return text

def build_output_text(filenames, page_delineation):

    text = ''
    ordered_filenames = []
    for i in range(0,len(filenames)):
        ordered_filenames.append('')
    for filename in filenames:
        index = int(filename.split('.pdf.tiff.txt')[0].split('-p')[-1])
        ordered_filenames[index] = filename
    for filename in ordered_filenames:
        with open(filename) as f:
            contents = f.read()
        text += '{0}{1}'.format(contents, page_delineation)

    return text

def cleanup_yapot(pdf_filename, text_filenames):

    try:
        for filename in text_filenames:
            os.remove(filename)
        os.remove('{0}_unsecured.pdf'.format(pdf_filename))
    except Exception as e:
        #print e
        pass




import os
import sys
import subprocess
from yapot import _convert_image_to_text

def _convert_to_tiff(image_filename):

    try:

        FNULL = open(os.devnull, 'w')

        output_filename = '{0}.tiff'.format(image_filename)
        cli_call = [
            'convert',
            image_filename,
            output_filename,
        ]
        subprocess.call(
            cli_call,
            stdout=FNULL,
            stderr=subprocess.STDOUT
        )
    except Exception, e:
        print "ERROR: {0}".format(e)

    return output_filename

def convert_image(image_filename):

    tiff_filename = _convert_to_tiff(image_filename)

    image_text = _convert_image_to_text(tiff_filename)

    #print image_text

    return image_text

if __name__ == '__main__':

    print "Welcome to yapot!"

    if len(sys.argv) != 3:
        print "Usage:\n\n\tpython img-conv.py <image_filename> <output_text_filename>\n\n"
    else:
        image_filename = sys.argv[1]
        text_filename = sys.argv[2]

        success, image_text = convert_image(image_filename)

        if success == True:
            with open(text_filename, 'w') as f:
                f.write(image_text)
            print "Done!"
        else:
            print "Image -> Text conversion was not successful."



import os
import sys
import uuid
from yapot import convert_document

def run():
    print("Welcome to yapot!")
    if len(sys.argv) != 2:
        print(("Usage:\n\n\t"
               "python cli-tool.py <pdf_filename> <output_dir>\n\n"))
    else:
        pdf_filename = sys.argv[1]
        #output_dir = sys.argv[2]
        #base_page_name = os.path.expanduser(pdf_filename)

        #if not os.path.exists(output_dir):
        #    os.makedirs(output_dir)

        temp_dir = '{0}'.format(str(uuid.uuid4()))
        if not os.path.exists(temp_dir):
            os.makedirs(temp_dir)

        pdf_text = convert_document(
            pdf_filename = pdf_filename,
            #base_page_name = base_page_name,
            resolution = 300,
            delete_files = True,
            page_delineation = '\n--------\n',
            verbose = True,
            temp_dir = temp_dir,
            #make_thumbs = True,
            #thumb_size = 512,
            #thumb_dir = '{0}/thumbs'.format(temp_dir),
            pool_count = 1,
        )

        with open('%s.txt' % pdf_filename, 'w') as f:
            f.write(pdf_text)

        print("Done.")

if __name__ == '__main__':

    run()
